/******************************************************
Library for 24c32 from bitbanding STM32F030
		by Sergi 10.02.2017
******************************************************/
#ifndef __i2c_soft
#define __i2c_soft


// Connect i2c_SDA to PORTB7, i2c_SCL to PORTB6

#ifndef __delay_us
#define __delay_us
void delay_us(volatile unsigned long nTime)
	{ 
 nTime=nTime<<2;
 while(nTime != 0)
 {nTime--;
	nTime++;
	nTime--;	
 }
 }; 
#endif
	
unsigned char reserv[32];

int i2c_send_start (void)
{
	//   SDA
	GPIOB->BSRR = GPIO_BSRR_BS_6;			   		// SCL  1
	GPIOB->MODER &=~  GPIO_MODER_MODER7;		//SDA  
	if (!(GPIOB->IDR & 0x80)) return 1;
	GPIOB->MODER |= GPIO_MODER_MODER7_0;		//SDA  
	GPIOB->BSRR = GPIO_BSRR_BR_7;				   	// SDA  0
	delay_us(10);
	GPIOB->BSRR = GPIO_BSRR_BR_6;				   	// SCL  0
	delay_us(10);
	return 0;
};

void i2c_send_stop (void)
{
	GPIOB->MODER |= GPIO_MODER_MODER7_0;		//SDA  
	GPIOB->BSRR = GPIO_BSRR_BR_6;			   		// SCL  0
	GPIOB->BSRR = GPIO_BSRR_BR_7;			  	 	// SDA  0
	delay_us(10);
	GPIOB->BSRR = GPIO_BSRR_BS_6;			  	 	// SCL  1
	delay_us(10);
	GPIOB->BSRR = GPIO_BSRR_BS_7;			   		// SDA  1
	delay_us(10);
};

void i2c_send_wr (unsigned char wr)
{
	unsigned char tmp = 0x80;
	while (tmp)
	{
		GPIOB->MODER |= GPIO_MODER_MODER7_0;				//SDA  
		delay_us(10);
		if (tmp & wr)GPIOB->BSRR=GPIO_BSRR_BS_7;		// SDA  1
		else GPIOB->BSRR = GPIO_BSRR_BR_7;			   	// SDA  0
		tmp = tmp>>1;
		delay_us(10);
		GPIOB->BSRR = GPIO_BSRR_BS_6;			  			 	// SCL  1
		delay_us(10);
		GPIOB->BSRR = GPIO_BSRR_BR_6;			   				// SCL  0
	};
	GPIOB->MODER &=~  GPIO_MODER_MODER7; 					//SDA  
	delay_us(10);
	GPIOB->BSRR = GPIO_BSRR_BS_6;			 				  	// SCL  1
	delay_us(10);
	GPIOB->BSRR = GPIO_BSRR_BR_6;			  				 	// SCL  0
	delay_us(10);
};

unsigned char i2c_send_rd (void)
{
	unsigned char tmp = 0x80,ret = 0;
	GPIOB->MODER &=~  GPIO_MODER_MODER7;		 //SDA  
	while (tmp)
	{
		GPIOB->BSRR = GPIO_BSRR_BS_6;			 		// SCL  1
		delay_us(10);
		if (GPIOB->IDR & 0x80) ret |= tmp;
		tmp = tmp>>1;
		delay_us(10);
		GPIOB->BSRR = GPIO_BSRR_BR_6;			   	// SCL  0
		delay_us(10);
	};
	GPIOB->BSRR = GPIO_BSRR_BS_6;			  	 	// SCL  1
	delay_us(10);
	GPIOB->BSRR = GPIO_BSRR_BR_6;			   		// SCL  0
	delay_us(10);														//NAK!
	return ret;
};

unsigned char i2c_read_data (int adrr)
{
	int tmp = 0;
	i2c_send_start();
	i2c_send_wr (0xA0);
	i2c_send_wr (adrr >> 8);  //for 24c32
	i2c_send_wr (adrr & 0xff);
	i2c_send_start();
	i2c_send_wr (0xA1);
	tmp = i2c_send_rd();
	i2c_send_stop();
	return tmp;
};

void i2c_write_data(int adrr,unsigned char data)
	{
	i2c_send_start();
	i2c_send_wr (0xA0);
	i2c_send_wr (adrr >> 8);  //for 24c32
	i2c_send_wr (adrr & 0xff);
	i2c_send_wr (data);
	i2c_send_stop();
	delay_us(50000);		
	};

void i2c_write_32(int adrr,long int data)
{
	i2c_write_data(adrr,(data >> 24));
	i2c_write_data(adrr + 1,((data >> 16) & 0xff));
	i2c_write_data(adrr + 2,((data >> 8) & 0xff));
	i2c_write_data(adrr + 3,(data & 0xff));
};

long int i2c_read_32(int adrr)
{
	long int tp = 0;
	tp = tp | ((long int)(i2c_read_data(adrr)) << 24);
	tp = tp | ((long int)(i2c_read_data(adrr + 1)) << 16);
	tp = tp | ((long int)(i2c_read_data(adrr + 2)) << 8);
	tp = tp | i2c_read_data(adrr + 3);
	return tp;
};

void save_const (int adrr)
{
	unsigned char i;
	i2c_send_start();
	i2c_send_wr (0xA0);
	i2c_send_wr (adrr >> 8);  //for 24c32
	i2c_send_wr (adrr & 0xff);
	for (i=0;i<32;i++)
	{
		i2c_send_wr (reserv[i]);
	};
	i2c_send_stop();
	delay_us(5000);	
};

void load_const (int adrr)
{
	unsigned char i;
	for (i=0;i<32;i++)
	{
	reserv[i] = i2c_read_data(i+adrr);
	};
};

#endif
